#pragma once

#include "AbstractEncoder.h"
#include "HuffmanEncoder.h"

#include <string>
#include <vector>
#include <memory>

class PrefixCodeGenerator;

class Lz77Encoder : public AbstractEncoder
{
public:
    using Buffer = std::vector<unsigned char>;

    Lz77Encoder(BitStream* inputStream, BitStream* outputStream);

    unsigned lookAheadForMatchingChars(std::vector<char>& matchBuffer, unsigned searchIndex, unsigned hitOffset, const std::string& stream, unsigned streamLoc);

    void lookThroughSearchBuffer(char searchChar, unsigned& hitLength, unsigned& hitOffset, const std::string& stream, unsigned streamLoc);

    bool encode() override;

    bool decode() override;

    void setSearchBufferSize(unsigned size);

    void setLookAheadBufferSize(unsigned size);

    void setPrefixCodeGenerator(std::unique_ptr<PrefixCodeGenerator> generator);

private:
    unsigned mSearchBufferSize{32000};
    Buffer mSearchBuffer;

    unsigned mLookAheadBufferSize{256};
    Buffer mLookaheadBuffer;

    std::unique_ptr<PrefixCodeGenerator> mCodeGenerator;
};
